// extensions used for object prototype

// attach the .compare method to Array's prototype to call it on any array
/*
Usage:

[1, 2, [3, 4]].compare([1, 2, [3, 2]]) === false;
[1, "2,3"].compare([1, 2, 3]) === false;
[1, 2, [3, 4]].compare([1, 2, [3, 4]]) === true;
[1, 2, 1, 2].compare([1, 2, 1, 2]) === true;
*/

Array.prototype.compare = function (array) {
    // if the other array is a falsy value, return
    if (!array)
        return false;

    // compare lengths - can save a lot of time
    if (this.length != array.length)
        return false;

    for (var i = 0; i < this.length; i++) {
        // Check if we have nested arrays
        if (this[i] instanceof Array && array[i] instanceof Array) {
            // recurse into the nested arrays
            if (!this[i].compare(array[i]))
                return false;
        }
        else if (this[i] != array[i]) {
            // Warning - two different object instances will never be equal: {x:20} != {x:20}
            return false;
        }
    }
    return true;
}